"use strict";

const fs = require("fs");
const path = require("path");
const _ = require("lodash");
const cheerio = require("cheerio");
const readdir = require("recursive-readdir");

let folder = path.join(__dirname, "..", "docs");
let fileMaping = {};

getFilesIn(folder).then((files) => {
	return files.reduce((promise, file) => {
		// Change links in files
		return readFile(file).then(contents => update(file, contents));
	}, Promise.resolve()).then(() => {
		// Add 100ms so that the file system has time to close all files
		// (else, we won't be able to rename them)
		return new Promise((resolve, reject) => setTimeout(resolve, 100));
	}).then(() => {
		// Rename files on disk
		return Promise.all(_.values(fileMaping)
			.map(({path, oldName, newName}) => renameFile(path, oldName, newName)));
	});
}).then(() => process.exit(0)).catch(error);

function error(err) {
	if(err.stack) console.error(err.stack);
	else console.error(err);
	process.exit(1);
}

function getFilesIn(folder) {
	return new Promise((resolve, reject) => {
		let ignoreFunc = (file, stats) => ! stats.isDirectory() && file.match(/\.html$/i) === null;
		readdir(folder, [ignoreFunc], function(err, files) {
			if(err) reject(err);
			else resolve(files);
		});
	});
}

function update(file, contents) {
	let promise = Promise.resolve();
	let $ = cheerio.load(contents);
	$(".tag-source a").each((idx, elem) => {
		let currentName = $(elem).attr("href");
		let match = currentName.match(/screeps-js-api.src.((.+.js.html)#?(.*))/i);
		if(match !== null) {
			let [_, link, fileName, anchor] = match;
			if(fileMaping[fileName] === undefined)
				fileMaping[fileName] = {
					path: path.dirname(file),
					oldName: currentName,
					newName: fileName
				};
			$(elem).attr("href", link);
			if(anchor.length > 0) $(elem).text(anchor);
			else $(elem).text(fileName);
		}
	});
	return promise.then(() => writeFile(file, $.html()));
}

function readFile(file) {
	return new Promise((resolve, reject) => {
		fs.readFile(file, (err, contents) => {
			if(err) reject(err);
			else resolve(contents);
		});
	});
}

function renameFile(folder, oldName, newName) {
	let oldPath = path.join(folder, oldName);
	let newPath = path.join(folder, newName);
	return new Promise((resolve, reject) => {
		fs.rename(oldPath, newPath, (err) => {
			if(err) reject(err);
			else resolve();
		});
	});
}

function writeFile(file, contents) {
	return new Promise((resolve, reject) => {
		fs.writeFile(file, contents, (err) => {
			if(err) reject(err);
			else resolve();
		});
	});
}
